use std::fs;
use std::io;

use hound;
use rand::prelude::*;

const SAMPLE_RATE : f32 = 44100.0;

fn main()
{
    let wav_spec : hound::WavSpec = hound::WavSpec
    {
        channels        : 1,
        sample_rate     : SAMPLE_RATE as u32,
        bits_per_sample : 32,
        sample_format   : hound::SampleFormat::Float
    };

    let mut wave_writer : hound::WavWriter<io::BufWriter<fs::File>> = hound::WavWriter::create("d:/noise.wav", wav_spec).unwrap();

    let mut wave_vector : Vec<f32> = vec![];

    let bpm : usize = 120;
 
    wave_vector.extend(get_noise_wave_vector(2.0, -1.0, calculate_note_length(bpm, 2)));
    wave_vector.extend(get_noise_wave_vector(0.2,  0.8, calculate_note_length(bpm, 2)));
    wave_vector.extend(get_noise_wave_vector(0.8, -1.0, calculate_note_length(bpm, 2)));
 
    for value in wave_vector.into_iter()
    {
        wave_writer.write_sample(value).unwrap();
    }
}

// bpm  : 분당 박자, beats per minute
// note : 음표 예) 4(4분 음표)
fn calculate_note_length(bpm : usize, note: usize) -> usize
{
    return ((4.0 / note as f32) * (60.0 / bpm as f32) * SAMPLE_RATE) as usize;
}

// range       : 범위
// shift       : 시프트
// note_length : 음표 길이 (샘플 수)
fn get_noise_wave_vector(range : f32, shift : f32, note_length : usize) -> Vec<f32>
{
    let mut wave_vector : Vec<f32> = vec![0.0; note_length];

    let mut thread_rng : ThreadRng = rand::thread_rng();

    for i in 0..note_length
    {
        wave_vector[i] = thread_rng.gen::<f32>() * range + shift;
    }
 
    let gain : f32 = 0.5;
 
    return wave_vector.into_iter().map(|v : f32| (v * gain) as f32).collect();
}